#!/usr/local/bin/perl

#use strict;
use warnings;

use JSON;
use Getopt::Long;

{
	my $separator = undef;
	my $show_help = 0;
	my $file = undef;
	GetOptions(
		"--separator|s=s" => \$separator,
		"--file=s" => \$file,
		"--help|h!" => \$show_help,
	) or show_help();
	show_help() if $show_help;

	## use STDIN if file name is not specified
	my ($fn) = @ARGV;
	unless (defined $fn) {
		$fn = "C:";
	}
	my $outFile = "C:/operations.json";
	if (defined $file) {
		$outFile = "$fn/$file";
	}
	
	my @Files = GetFilesList ($fn);
	my @data;

	foreach my $fichier (@Files) {
		if ($fichier =~ m{\.csv$}) {
			read_csv(\@data, $fichier, $separator);
		}
	}
	
	my $output = \*STDOUT;
	open($output, ">", $outFile) or die "Ecriture impossible de [$outFile]: $!";
	write_json(\@data, $output);
	close($output);
}


sub show_help {
	print "Usage: $0 [--separator=?] [--file=jsonfile] [--help] <directory>\n";
	exit(1);
}

sub GetFilesList
{
        my $Path = $_[0];
        my $FileFound;
        my @FilesList=();

        # Lecture de la liste des fichiers
        opendir (my $FhRep, $Path)
                or die "Impossible d'ouvrir le repertoire $Path\n";
        my @Contenu = grep { !/^\.\.?$/ } readdir($FhRep);
        closedir ($FhRep);

        foreach my $FileFound (@Contenu) {
                # Traitement des fichiers
                if ( -f "$Path/$FileFound") {
                        push ( @FilesList, "$Path/$FileFound" );
                }
                # Traitement des repertoires
                elsif ( -d "$Path/$FileFound") {
                        # Boucle pour lancer la recherche en mode recursif
                        push (@FilesList, GetFilesList("$Path/$FileFound") );
                }

        }
        return @FilesList;
}



sub write_json {
	my ($data, $output) = @_;

	print $output to_json(
		$data, 
		{
			'latin1' => 1, 
			'pretty' => 1,
		}
	);
}



sub read_csv {
	my ($data, $fn, $separator) = @_;
	
	# my @lignes;
	my @entetes = ("Date d'opération", "Date de valeur", "Type", "Carte bancaire", "Chèque ou Référence", "Montant total", "Tiers", "Ventilation");
	my @cles = ("Catégorie", "Sous-catégorie", "Montant", "Notes");
	my @champs;
	my $numligne = 0;

	open(my $fh, "<", $fn) or die "Impossible d'ouvrir [$fn]: $!";

	while (defined($ligne = <$fh>)) {
		chomp $ligne;
	
		my %nvLigne;
		
		if ($numligne > 0) {
			@champs = split($separator, $ligne);
			
			my @ventilation;
			
			if ($champs[7] =~ m/\|/) {
				my @categories = split(/\|/, $champs[7]);
				my @reste = split(/\|/, $champs[8]);
				my $numCategorie = 0;
				
				foreach my $categorie (@categories) {
					my %affectation;
					my $notes;
					my $sousCategorie;
					
					if ($reste[$numCategorie] =~ m/\//) {
						my @groupe = split(/\//, $reste[$numCategorie]);
						$sousCategorie = $groupe[0];
						$notes = $groupe[1];
					}
					else {
						$sousCategorie = $reste[$numCategorie];
						$notes = "";
					}
					
					my @valeurs = ($categorie, $sousCategorie, $reste[$numCategorie + 1] + 0, $notes);
					
					@affectation{@cles} = @valeurs;
					push(@ventilation, \%affectation);
					
					$numCategorie = $numCategorie + 2;
				}
			}
			else {
				my %affectation;
				my @valeurs = ($champs[7], $champs[8], $champs[5] + 0, $champs[9]);
				
				@affectation{@cles} = @valeurs;
				push(@ventilation, \%affectation);
			}
			
			@nvLigne{@entetes} = ($champs[0], $champs[1], $champs[2], $champs[3], $champs[4], $champs[5] + 0, $champs[6], \@ventilation);
			push(@$data, \%nvLigne);
		}
		
		$numligne++;
	}

	close($fh); 
	
	# return \@lignes;
}



